/*
 *                                                                                                  geany_encoding=koi8-r
 * usb.h
 *
 * Copyright 2018 Edward V. Emelianov <eddy@sao.ru, edward.emelianoff@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 */
#pragma once
#ifndef __USB_H__
#define __USB_H__

#include "hardware.h"

// send string with constant length
#define USND(idx, str)  do{USB_send(idx, (uint8_t*)str, sizeof(str)-1);}while(0)
extern uint8_t ifacesno, errcode;
extern uint8_t dontans[8];

void USB_setup();
void usb_proc();
void USB_send(uint8_t idx, const uint8_t *buf, uint16_t len);
void USB_sendstr(uint8_t idx, const char *str);
void USB_send_blk(uint8_t idx, const uint8_t *buf, uint16_t len);
uint8_t USB_receive(uint8_t idx, uint8_t *buf);

#endif // __USB_H__
